/*
 * mxallowd
 * (c) 2007-2008 Michael Stapelberg
 * 
 * See mxallowd.c for description, website and license information
 *
 */

#ifndef _CONFIG_INCLUDED
#define _CONFIG_INCLUDED
bool isInteger(char*);
void getNextConfigEntry(FILE*, char**, char**, char*, int);
bool checkConfigValue(char*, char*);
bool read_configuration(char*);
#endif
