/*
 * mxallowd
 * (c) 2007-2008 Michael Stapelberg
 * 
 * See mxallowd.c for description, website and license information
 *
 */

#ifndef _LOG_INCLUDED
#define _LOG_INCLUDED
void slog(char*, ...);
void slogerror(char*);
void diem(char *message);
void /*@noreturn@*/ diep(char *message);
void /*@noreturn@*/ dief(char *format, ...);
#endif
