/*
 * mxallowd
 * (c) 2007-2008 Michael Stapelberg
 * 
 * See mxallowd.c for description, website and license information
 *
 */

#ifndef _RESOLVE_INCLUDED
#define _RESOLVE_INCLUDED
char *rdns_for_ip(char*);
struct hostent *ip_for_dns(char*);
void *resolve_thread(void*);
#endif
